@@ -825,37 +825,6 @@ rbg_inspect (VALUE object)
     return StringValueCStr(inspected);
 }
 
-VALUE
-rbg_to_array (VALUE object)
-{
-    return rb_convert_type(object, RUBY_T_ARRAY, "Array", "to_ary");
-}
-
-VALUE
-rbg_to_hash (VALUE object)
-{
-    return rb_convert_type(object, RUBY_T_HASH, "Hash", "to_hash");
-}
-
-VALUE
-rbg_check_array_type (VALUE object)
-{
-#ifdef HAVE_RB_CHECK_ARRAY_TYPE
-    return rb_check_array_type(object);
-#else
-    return rb_check_convert_type(object, RUBY_T_ARRAY, "Array", "to_ary");
-#endif
-}
-
-VALUE
-rbg_check_hash_type (VALUE object)
-{
-#ifdef HAVE_RB_CHECK_HASH_TYPE
-    return rb_check_hash_type(object);
-#else
-    return rb_check_convert_type(object, RUBY_T_HASH, "Hash", "to_hash");
-#endif
-}
 
 void
 rbg_scan_options (VALUE options, ...)
@@ -866,7 +835,7 @@ rbg_scan_options (VALUE options, ...)
     VALUE *value;
     va_list args;
 
-    options = rbg_check_hash_type(options);
+    options = rb_check_convert_type(options, T_HASH, "Hash", "to_hash");
     if (NIL_P(options)) {
         options = rb_hash_new();
     } else if (options == original_options) {
