@@ -103,16 +103,12 @@ module Test
       end
 
       def inspect_tag(tag)
-        if jruby?
-          "`#{tag}'".inspect
-        else
-          begin
-            throw tag
-          rescue NameError
-            tag.to_s.inspect
-          rescue ArgumentError
-            tag.inspect
-          end
+        begin
+          throw tag
+        rescue NameError
+          tag.to_s.inspect
+        rescue ArgumentError
+          tag.inspect
         end
       end
 
@@ -593,24 +589,49 @@ EOM
         check_nothing_fails {
           assert_instance_of(String, "string", "successful assert_instance_of")
         }
-        check_fail(%Q{<"string"> expected to be an instance of\n<Hash> but was\n<String>.}) {
+        check_fail(%Q{<"string"> expected to be instance_of?\n<Hash> but was\n<String>.}) {
           assert_instance_of(Hash, "string")
         }
-        check_fail(%Q{failed assert_instance_of.\n<"string"> expected to be an instance of\n<Hash> but was\n<String>.}) {
+        check_fail(%Q{failed assert_instance_of.\n<"string"> expected to be instance_of?\n<Hash> but was\n<String>.}) {
           assert_instance_of(Hash, "string", "failed assert_instance_of")
         }
 
         check_nothing_fails do
           assert_instance_of([Fixnum, NilClass], 100)
         end
-        check_fail(%Q{<"string"> expected to be an instance of\n[<Fixnum>, <NilClass>] but was\n<String>.}) do
+        check_fail(%Q{<"string"> expected to be instance_of?\n[<Fixnum>, <NilClass>] but was\n<String>.}) do
           assert_instance_of([Fixnum, NilClass], "string")
         end
-        check_fail(%Q{<100> expected to be an instance of\n[<Numeric>, <NilClass>] but was\n<Fixnum>.}) do
+        check_fail(%Q{<100> expected to be instance_of?\n[<Numeric>, <NilClass>] but was\n<Fixnum>.}) do
           assert_instance_of([Numeric, NilClass], 100)
         end
       end
 
+      def test_assert_not_instance_of
+        check_nothing_fails {
+          assert_not_instance_of(NilClass, "string")
+        }
+        check_nothing_fails {
+          assert_not_instance_of(NilClass, "string", "successful assert_instance_of")
+        }
+        check_fail(%Q{<"string"> expected to not be instance_of?\n<String> but was.}) {
+          assert_not_instance_of(String, "string")
+        }
+        check_fail(%Q{failed assert.\n<"string"> expected to not be instance_of?\n<String> but was.}) {
+          assert_not_instance_of(String, "string", "failed assert")
+        }
+
+        check_nothing_fails do
+          assert_not_instance_of([Numeric, NilClass], 100)
+        end
+        check_fail(%Q{<100> expected to not be instance_of?\n[<Fixnum>, <NilClass>] but was.}) do
+          assert_not_instance_of([Fixnum, NilClass], 100)
+        end
+        check_fail(%Q{<"str"> expected to not be instance_of?\n[<Numeric>, <String>] but was.}) do
+          assert_not_instance_of([Numeric, String], 'str')
+        end
+      end
+
       def test_assert_nil
         check_nothing_fails {
           assert_nil(nil)
@@ -628,14 +649,14 @@ EOM
           assert_nil("string", "failed assert_nil")
         }
       end
-      
+
       def test_assert_not_nil
         check_nothing_fails{assert_not_nil(false)}
         check_nothing_fails{assert_not_nil(false, "message")}
         check_fail("<nil> expected to not be nil."){assert_not_nil(nil)}
         check_fail("message.\n<nil> expected to not be nil.") {assert_not_nil(nil, "message")}
       end
-        
+
       def test_assert_kind_of
         check_nothing_fails {
           assert_kind_of(Module, Array)
@@ -644,7 +665,7 @@ EOM
           assert_kind_of(Object, "string", "successful assert_kind_of")
         }
         check_nothing_fails {
-          assert_kind_of(Object, "string", "successful assert_kind_of")
+          assert_kind_of(String, "string", "successful assert_kind_of")
         }
         check_nothing_fails {
           assert_kind_of(Comparable, 1)
@@ -664,6 +685,31 @@ EOM
         end
       end
 
+      def test_assert_not_kind_of
+        check_nothing_fails {
+          assert_not_kind_of(Class, 42)
+        }
+        check_nothing_fails {
+          assert_not_kind_of(Symbol, "string", "successful assert_not_kind_of")
+        }
+        check_nothing_fails {
+          assert_not_kind_of(Integer, 1.1)
+        }
+        check_fail(%Q{<1> expected to not be kind_of?\n<Integer> but was.}) {
+          assert_not_kind_of(Integer, 1)
+        }
+        check_fail(%Q{failed assert_not_kind_of.\n<"string"> expected to not be kind_of?\n<String> but was.}) {
+          assert_not_kind_of(String, "string", "failed assert_not_kind_of")
+        }
+
+        check_nothing_fails do
+          assert_not_kind_of([String, NilClass], 100)
+        end
+        check_fail(%Q{<100> expected to not be kind_of?\n[<Fixnum>, <NilClass>] but was.}) do
+          assert_not_kind_of([Fixnum, NilClass], 100)
+        end
+      end
+
       def test_assert_match
         check_nothing_fails {
           assert_match(/strin./, "string")
@@ -687,7 +733,7 @@ EOM
           assert_match(/slin./, "string", "failed assert_match")
         }
       end
-      
+
       def test_assert_same
         thing = "thing"
         check_nothing_fails {
@@ -707,7 +753,7 @@ EOM
           assert_same(thing, thing2, "failed assert_same")
         }
       end
-      
+
       def test_assert_nothing_raised
         check_nothing_fails {
           assert_nothing_raised {
@@ -797,7 +843,7 @@ EOM
           assert_not_same(thing, thing, "message")
         }
       end
-      
+
       def test_assert_not_equal
         check_nothing_fails {
           assert_not_equal("string1", "string2")
@@ -828,7 +874,7 @@ EOM
       def test_assert_not_match_fail_not_regexp
         check_fail("<REGEXP> in assert_not_match(<REGEXP>, ...) " +
                     "should be a Regexp.\n" +
-                    "<\"asdf\"> expected to be an instance of\n" +
+                    "<\"asdf\"> expected to be instance_of?\n" +
                     "<Regexp> but was\n" +
                     "<String>.") do
           assert_not_match("asdf", "asdf")
@@ -853,7 +899,7 @@ EOM
       def test_assert_no_match
         check_nothing_fails{assert_no_match(/sling/, "string")}
         check_nothing_fails{assert_no_match(/sling/, "string", "message")}
-        check_fail(%Q{The first argument to assert_no_match should be a Regexp.\n<"asdf"> expected to be an instance of\n<Regexp> but was\n<String>.}) do
+        check_fail(%Q{The first argument to assert_no_match should be a Regexp.\n<"asdf"> expected to be instance_of?\n<Regexp> but was\n<String>.}) do
           assert_no_match("asdf", "asdf")
         end
         check_fail(%Q{</string/> expected to not match\n<"string">.}) do
@@ -886,7 +932,7 @@ EOM
           end
         end
       end
-      
+
       def test_assert_nothing_thrown
         check_nothing_fails do
           assert_nothing_thrown("message") do
@@ -903,7 +949,7 @@ EOM
           end
         end
       end
-      
+
       def test_assert_operator
         check_nothing_fails {
           assert_operator("thing", :==, "thing", "message")
@@ -916,6 +962,18 @@ EOM
         }
       end
 
+      def test_assert_not_operator
+        check_nothing_fails {
+          assert_not_operator("thing", :==, "Thing", "message")
+        }
+        check_fail(%Q{<42>\ngiven as the operator for #assert_not_operator must be a Symbol or #respond_to?(:to_str).}) do
+          assert_not_operator("thing", 42, "message")
+        end
+        check_fail(%Q{message.\n<0> expected to not be\n==\n<0.0>.}) {
+          assert_not_operator(0, :==, 0.0, "message")
+        }
+      end
+
       def test_assert_respond_to
         check_nothing_fails {
           assert_respond_to("thing", :to_s, "message")
